#include <Rcpp.h>
#include <stdio.h>
using namespace Rcpp;

// This is a simple example of exporting a C++ function to R. You can
// source this function into an R session using the Rcpp::sourceCpp 
// function (or via the Source button on the editor toolbar). Learn
// more about Rcpp at:
//
//   http://www.rcpp.org/
//   http://adv-r.had.co.nz/Rcpp.html
//   http://gallery.rcpp.org/
//

bool nextCombination(int n, int r, int *a) {
  int lastNotEqualOffset = r-1;
  while (a[lastNotEqualOffset] == n-r+(lastNotEqualOffset+1)) {
    lastNotEqualOffset--;
  }
  if (lastNotEqualOffset < 0) {
    return false;
  }
  a[lastNotEqualOffset]++;
  for (int i = lastNotEqualOffset+1; i<r; i++) {
    a[i] = a[lastNotEqualOffset]+(i-lastNotEqualOffset);
  }
  return true;
}

// [[Rcpp::export]]
NumericVector EFMDR_RCPP(NumericMatrix data, double k_order) {
  double fake_zero = 0.00001;
  
  int n_sample = data.nrow();
  int n_SNP = data.ncol() - 1; //col_0: class(y)
  int index_combi[5] = {0};     //limited to 5-locus interaction
  int best_index_combi[5] = {0};     //limited to 5-locus interaction
  int dof = 1;
  double best_BA = 0;
  //int temp=0;
  
  NumericVector result(5);
  
  for(int i=0;i<k_order;i++){//initializing combination
    index_combi[i] = i+1;
    dof *= 3;
  }
  
  int* genotype = (int *)malloc(sizeof(int)*n_sample);
  double* contingency_0 = (double *)malloc(dof*sizeof(double));
  double* contingency_1 = (double *)malloc(dof*sizeof(double));
  
  do{     //searching all combinations
    //temp = index_combi[0];
    for(int i=0; i<dof; i++){      //init
      contingency_0[i] = fake_zero;
      contingency_1[i] = fake_zero;
    }
    
    for(int i=0; i<n_sample; i++){  //contingeny table build
      genotype[i]=0;
      for(int j=0; j<k_order; j++){
        genotype[i] = 3*genotype[i] + data(i,index_combi[j]);
      }
      if(0 == data(i,0)){
        contingency_0[genotype[i]]++;
      }else{
        contingency_1[genotype[i]]++;
      }
    }
    
    double TP, TN, FP, FN, mu_H, mu_L;
    TP=TN=FP=FN=0;
    for(int i=0; i<dof; i++){
      mu_H = contingency_1[i]/(contingency_0[i]+contingency_1[i]);
      mu_L = 1-mu_H;
      TP += contingency_1[i]*mu_H;
      FN += contingency_1[i]*mu_L;
      FP += contingency_0[i]*mu_H;
      TN += contingency_0[i]*mu_L;
    }
    double BA = (TP/(TP+FN)+TN/(TN+FP))/2;
    if(best_BA < BA){
      best_BA = BA;
      for(int i=0; i<k_order;i++){
        best_index_combi[i] = index_combi[i];
      }
    }
  } while (nextCombination(n_SNP, k_order, index_combi));
  
  for(int i=0; i<k_order;i++){
    result(i) = best_index_combi[i];
  }

  return result;
}
