################  R source code of EFMDR  ######################
# by Sangseob Leem
#e-mail: leemss@snu.ac.kr

# main function
EFMDR <- function(data, k){
  fake_zero = 0.00001
  SNP_combinations = combn(2:ncol(data), k)
  best_index = 0
  best_BA = 0
  dof = 0
  for(i in 1: ncol(SNP_combinations)){
    data$genotype <- 0
    for(j in 1: k){
      data[,"genotype"] = data[,"genotype"]*3 + data[,SNP_combinations[j,i]]
    }
    contingency_table <- table(data[,1], data[,"genotype"])+fake_zero
    dof = ncol(contingency_table)-1
    TP = 0
    TN = 0
    FP = 0
    FN = 0
    for(j in 1:ncol(contingency_table)){
      mu_H = contingency_table[2,j]/(contingency_table[1,j]+contingency_table[2,j])
      mu_L = 1-mu_H
      TP = TP + contingency_table[2,j]*mu_H
      FN = FN + contingency_table[2,j]*mu_L
      FP = FP + contingency_table[1,j]*mu_H
      TN = TN + contingency_table[1,j]*mu_L
    }
    BA = (TP/(TP+FN)+TN/(FP+TN))/2
    if(best_BA < BA){
      best_BA = BA
      best_index = i
    }
  }
  chi_square_value = 2*best_BA*nrow(data)-nrow(data)
  p_value = 1-pchisq(chi_square_value,dof)
  
  result <-c()
  result$SNP <- SNP_combinations[,best_index]-1
  result$p_value <- p_value
  result$BA_fuzzy <- best_BA
  data$genotype <- 0
  for(j in 1: k){
    data[,"genotype"] = data[,"genotype"]*3 + data[,SNP_combinations[j,best_index]]
  }
  contingency_table <- table(data[,1], data[,"genotype"])
  result$contingency_table <- contingency_table
  
  p_case = nrow(subset(data,data[,1] == 1))/nrow(data)
  p_control = nrow(subset(data,data[,1] == 0))/nrow(data)
  
  x_num = 3^as.integer((k+1)/2)
  y_num = 3^as.integer(k/2)
  
  for(i in 1:y_num){
    for(j in 1:x_num){
      index = (i-1)*x_num + j-1
      if(index %in% variable.names(result$contingency_table)){
        #p_cell_temp = pbinom(result$contingency_table[2,as.character(index)]-1, size=(result$contingency_table[2,as.character(index)]+result$contingency_table[1,as.character(index)]),                             prob=p_case)                     +pbinom(result$contingency_table[2,as.character(index)], size=(result$contingency_table[2,as.character(index)]+result$contingency_table[1,as.character(index)]),                              prob=p_case)
        #p_cell_temp = p_cell_temp/2
        n_case = result$contingency_table[2,as.character(index)]
        n_control = result$contingency_table[1,as.character(index)]
        e_case = (n_case+n_control)*p_case
        e_control = (n_case+n_control)*(1-p_case)
        chi_temp = (n_case-e_case)^2/e_case+(n_control-e_control)^2/e_control
        p_cell_temp = 1-pchisq(chi_temp, df=1)
        p_cell_temp=sqrt(p_cell_temp)
        if(n_case>e_case){
          p_cell_temp = 1-p_cell_temp/2
        }else{
          p_cell_temp = p_cell_temp/2
        }
      }
      else{
        p_cell_temp = 0.5
      }
      result$p_cell<- c(result$p_cell,1-p_cell_temp)
    }
  }
  return(result)
}

# function to draw fuzzy MDR plot
draw_cell <- function(x_base, y_base, x_size, y_size, n_cases, n_controls, y_scale, back_col){
  rect(x_base, y_base, x_base+x_size, y_base+y_size, col = back_col)   #border line
  rect(x_base+0.15*x_size, y_base, x_base+0.4*x_size , y_base+n_cases/y_scale*y_size, col="white")   #cases
  rect(x_base+0.6*x_size , y_base, x_base+0.85*x_size, y_base+n_controls/y_scale*y_size, col="white")   #controls
  text(x_base+0.27*x_size, y_base+n_cases/y_scale*y_size+y_size*0.1, labels = n_cases)
  text(x_base+0.72*x_size, y_base+n_controls/y_scale*y_size+y_size*0.1, labels = n_controls)
}
text_x <- function(index, x_base, y_base, x_size, y_size, y_height, index_SNP){
  num_repeat <- 3^(index-1)
  for(i in 1: num_repeat){
    text(x_base+(i-1)*x_size/num_repeat+0.5 *x_size/num_repeat, y_base+0.8*y_size, labels = paste("SNP", index_SNP, sep = ""))
    text(x_base+(i-1)*x_size/num_repeat+0.16*x_size/num_repeat, y_base+0.3*y_size, labels = paste(LETTERS[index+y_height], LETTERS[index+y_height], sep = ""))
    text(x_base+(i-1)*x_size/num_repeat+0.5 *x_size/num_repeat, y_base+0.3*y_size, labels = paste(LETTERS[index+y_height], letters[index+y_height], sep = ""))
    text(x_base+(i-1)*x_size/num_repeat+0.84*x_size/num_repeat, y_base+0.3*y_size, labels = paste(letters[index+y_height], letters[index+y_height], sep = ""))
  }
}
text_y <- function(index, x_base, y_base, x_size, y_size, index_SNP){
  num_repeat <- 3^(index-1)
  for(i in 1: num_repeat){
    text(x_base+0.2*x_size,y_base+(i-1)*y_size/num_repeat+0.5 *y_size/num_repeat, labels = paste("SNP", index_SNP, sep = ""), srt=90)
    text(x_base+0.7*x_size,y_base+(i-1)*y_size/num_repeat+0.84*y_size/num_repeat, labels = paste(LETTERS[index], LETTERS[index], sep = ""))
    text(x_base+0.7*x_size,y_base+(i-1)*y_size/num_repeat+0.5 *y_size/num_repeat, labels = paste(LETTERS[index], letters[index], sep = ""))
    text(x_base+0.7*x_size,y_base+(i-1)*y_size/num_repeat+0.16*y_size/num_repeat, labels = paste(letters[index], letters[index], sep = ""))
  }
}
plot_EFMDR <- function(MDR_result, k){
  library(plotrix)
  x_range = c(0,1)
  y_range = c(0,1)
  par(mar=c(0, 0, 0, 0), oma=c(0,0,0,0))
  plot(x_range, y_range)
  x_label_size = 0.15
  y_label_size = 0.1
  
  gap_ratio = 0.2 
  
  x_num = 3^as.integer((k+1)/2)
  y_num = 3^as.integer(k/2)
  
  num_x_gap = x_num/3
  num_y_gap = y_num/3
  
  y_scale = max(MDR_result$contingency_table)*1.2
  
  x_size = (1-y_label_size)/(x_num+num_x_gap*gap_ratio)
  y_size = (1-x_label_size)/(y_num+num_y_gap*gap_ratio)
  
  #colfunc <- colorRampPalette(c("red","white", "green"), interpolate = "spline")
  colfunc <- colorRampPalette(c("red","white", "green"), bias=1, interpolate = "spline")
  my_col <- colfunc(1000)
  colfunc <- colorRampPalette(c("red",my_col[300],my_col[400],"white",my_col[600],my_col[700], "green"), bias=1, interpolate = "spline")
  my_col <- colfunc(1000)
  
  x_height <- as.integer((k+1)/2)
  y_height <- as.integer(k/2)
  
  gradient.rect(0,1,y_label_size/5,0.7,col = my_col,gradient="y")
  text(y_label_size/3,1,   "H")
  text(y_label_size/3,0.7, "L")
  
  for(i in 1:y_height){
    text_y(index = i, x_base =  (i-1)*y_label_size/y_height, y_base = 0,
           x_size = y_label_size/y_height, y_size = 1-x_label_size, index_SNP = MDR_result$SNP[i])  ## x label
  }
  for(i in 1:x_height){
    text_x(index = i, x_base =  y_label_size, y_base = 1-i/x_height*x_label_size,
           x_size = 1-y_label_size, y_size = x_label_size/x_height, y_height = y_height, index_SNP = MDR_result$SNP[i+y_height])  ## x label
  }
  
  for(i in 1:x_num){
    for(j in 1:y_num){
      index = (j-1)*x_num + i-1
      if(index %in% variable.names(MDR_result$contingency_table)){
        draw_cell(x_base=y_label_size+(i-1+as.integer((i-1)/3)*gap_ratio)*x_size, y_base=1-x_label_size-(j+as.integer((j-1)/3)*gap_ratio)*y_size, x_size = x_size, y_size=y_size,
                  n_cases=MDR_result$contingency_table[2,as.character(index)],
                  n_controls = MDR_result$contingency_table[1,as.character(index)],
                  y_scale = y_scale, back_col = my_col[round(999*MDR_result$p_cell[index+1])+1])
      }
      else{
        draw_cell(x_base=y_label_size+(i-1)*x_size, y_base=1-x_label_size-j*y_size, x_size = x_size, y_size=y_size,
                  n_cases = 0, n_controls = 0,
                  y_scale = y_scale, back_col = my_col[500])
      }
    }
  }
}


# an example 
library(MDR)
data("mdr1")

res<-EFMDR(mdr1,2)
plot_EFMDR(res,2)

library(Rcpp)
sourceCpp("E:/CPP/EFMDR.cpp")

res2 <- EFMDR_RCPP(as.matrix(mdr1),2)
res_temp <- EFMDR(mdr1[,c(1,5,10)],2)

temp_mdr <- mdr1
for(i in 1:1000){
  temp_mdr[,ncol(temp_mdr)+1] <- NA
  temp_mdr[,ncol(temp_mdr)] = sample.int(3, nrow(mdr1), replace = T)-1
}


## 2-locus ##
start_time <- Sys.time()
res2 <-EFMDR(temp_mdr,2)
end_time <- Sys.time()
time2 <- end_time - start_time

start_time <- Sys.time()
res2_cpp <- EFMDR_RCPP(as.matrix(temp_mdr),2)
end_time <- Sys.time()
time2_cpp <- end_time - start_time

## 3-locus ## (It takes about two days.)
start_time <- Sys.time()
res3 <-EFMDR(temp_mdr,3)
end_time <- Sys.time()
time3 <- end_time - start_time

start_time <- Sys.time()
res3_cpp <- EFMDR_RCPP(as.matrix(temp_mdr),3)
end_time <- Sys.time()
time3_cpp <- end_time - start_time

## 4-locus ## (Do not run!! Maybe it will take more than 511 days to run this.)
start_time <- Sys.time()
res4 <-EFMDR(temp_mdr,4)
end_time <- Sys.time()
time4 <- end_time - start_time

start_time <- Sys.time()
res4_cpp <- EFMDR_RCPP(as.matrix(temp_mdr),4)
end_time <- Sys.time()
time4_cpp <- end_time - start_time
